#!/bin/sh 
###########
#VARIABLES
###########
SCRIPTNAME="zz_fix_bad_km"
MODULELIST="ipx irda x25 ax25 bluez bluetooth appletalk sctp pppoe pppox"
SCRIPTBASE="/usr/share/$SCRIPTNAME"
SAFEDIR=$SCRIPTBASE/safedir
LOGFILE=$SCRIPTBASE/$SCRIPTNAME.log
TMPLOGFILE=$SCRIPTBASE/$SCRIPTNAME.tmplog
MAILLIST="root"
AFFECTED="false"
REMOVED="false"

#First try to remove the modules, starting gently, then more aggressivly
echo "***************" >> $TMPLOGFILE
echo "Checking for affected modules already loaded" >> $TMPLOGFILE
echo "***************" >> $TMPLOGFILE
for i in $MODULELIST; do
	if `/sbin/lsmod |grep -q $i` ; then
		#echo "  $i: module is currently loaded in kernel"
		echo "  $i: module is currently loaded in kernel" >> $TMPLOGFILE
		AFFECTED="true"
		REMOVED="false"
		echo "*******************************" >> $TMPLOGFILE
		echo "Attempting to remove module $i" >> $TMPLOGFILE
		if [ "$REMOVED" = "false" ] ; then
			case "$i" in
				"bluez"|"bluetooth")
					/etc/init.d/hidd stop >> $TMPLOGFILE 2>&1  ;
					/etc/init.d/bluetooth stop >> $TMPLOGFILE 2>&1  ;
					;;
				"irda")
					/etc/init.d/irda stop >> $TMPLOGFILE 2>&1 ;
					;;
			esac
			if `/sbin/lsmod |grep -q $i` ; then
				echo "  Running shutdown scripts : Unsuccessful" >> $TMPLOGFILE
			else
				echo "  Running shutdown scripts : Successful" >> $TMPLOGFILE
				REMOVED="true"
			fi
		fi
		if [ "$REMOVED" = "false" ] ; then
			/sbin/modprobe -r $i >> $TMPLOGFILE 2>&1
			if `/sbin/lsmod |grep -q $i` ; then
				echo "  /sbin/modprobe -r $i : Unsuccessful" >> $TMPLOGFILE
			else
				echo "  /sbin/modprobe -r $i : Successful" >> $TMPLOGFILE
				REMOVED="true"
			fi
		fi
		if [ "$REMOVED" = "false" ] ; then
			/sbin/rmmod $i >> $TMPLOGFILE 2>&1
			if `/sbin/lsmod |grep -q $i` ; then
				echo "  /sbin/rmmod $i : Unsuccessful" >> $TMPLOGFILE
			else
				echo "  /sbin/rmmod $i : Successful" >> $TMPLOGFILE
				REMOVED="true"
			fi
		fi
		#if [ "$REMOVED" = "false" ] ; then
		#	/sbin/rmmod -f $i >> $TMPLOGFILE 2>&1
		#	if `/sbin/lsmod |grep -q $i` ; then
		#		echo "  /sbin/rmmod -f $i : Unsuccessful" >> $TMPLOGFILE
		#	else
		#		echo "  /sbin/rmmod -f $i : Successful" >> $TMPLOGFILE
		#		REMOVED="true"
		#	fi
		#fi
		if [ "$REMOVED" = "false" ] ; then
			echo "..."
			echo "We were unable to remove module $i from the kernel"
			echo "The machine will have to be rebooted in order to remove module $i"
			echo "This script will not reboot your machine, so you will have to do that manually"
			echo "We were unable to remove module $i from the kernel" >> $TMPLOGFILE
			echo "The machine will have to be rebooted in order to remove module $i" >> $TMPLOGFILE
			echo "This script will not reboot your machine, so you will have to do that manually" >> $TMPLOGFILE
		fi
		echo "*******************************" >> $TMPLOGFILE
	else
		#echo "  $i: passed - Not loaded in kernel"
		echo "  $i: passed - module is not loaded in kernel" >> $TMPLOGFILE
	fi
done


echo "  " >> $TMPLOGFILE
echo "***************" >> $TMPLOGFILE
echo "Moving modules so they wont be loaded" >> $TMPLOGFILE
echo "at some later date" >> $TMPLOGFILE
echo "***************" >> $TMPLOGFILE
rls=`lsb_release -rs`
[ ${rls#3} != $rls ] && sfx=o || sfx=ko
for i in $MODULELIST; do
    for j in `find /lib/modules/ -name $i.$sfx`; do
		kernel=`echo $j | cut -d'/' -f4`
		if ! [ -d $SAFEDIR/$kernel ] ; then
			/bin/mkdir -p $SAFEDIR/$kernel
		fi
		#echo "/bin/mv -vf $j $SAFEDIR/$kernel/" >> $TMPLOGFILE
		/bin/mv -vf $j $SAFEDIR/$kernel/ >> $TMPLOGFILE
    done
done


/bin/mail -s "$SCRIPTNAME Output" $MAILLIST < $TMPLOGFILE

cat $TMPLOGFILE >> $LOGFILE
/bin/rm -f $TMPLOGFILE

exit 0
